@@ -17,7 +17,7 @@ def main():
             DATABASES={'default': {'ENGINE': 'django.db.backends.sqlite3'}},
             ROOT_URLCONF = 'sitetree.tests',
             MIDDLEWARE_CLASSES=global_settings.MIDDLEWARE_CLASSES,  # Prevents Django 1.7 warning.
-            TEMPLATE_CONTEXT_PROCESSORS=global_settings.TEMPLATE_CONTEXT_PROCESSORS + (
+            TEMPLATE_CONTEXT_PROCESSORS=tuple(global_settings.TEMPLATE_CONTEXT_PROCESSORS) + (
                 'django.core.context_processors.request',
             )
         )
